#include "item.h"

#include "relay.h"
#include "../microcontroller.h"
#include "../actors/sensoractor.h"
#include "../actors/regulator.h"
#include "../actors/polynomalactor.h"

#include <QJsonArray>

ItemData::ItemData(uint32_t itemIdIn, QString name, uint8_t value): name_(name), value_(value), itemId_(itemIdIn)
{

}

QString ItemData::getName() const
{
    return name_;
}

void ItemData::setName(QString name)
{
    name_ = name;
}

uint8_t ItemData::getValue() const
{
    return value_;
}

uint32_t ItemData::id() const
{
    return  itemId_;
}


//item

bool Item::secondaryFlag = false;

Item::Item(uint32_t itemIdIn, QString name, uint8_t value,  QObject *parent): QObject(parent), ItemData (itemIdIn, name, value)
{

}

Item::Item(const ItemData& itemData,  QObject *parent): QObject(parent), ItemData(itemData)
{

}

Item::~Item()
{
    for(size_t i = 0; i < actors_.size(); i++) delete actors_[i];
}

void Item::store(QJsonObject &json)
{
    json["Name"] = name_;
    json["ItemId"] = static_cast<double>(itemId_);
    QJsonArray actorsArray;
    for(size_t i = 0; i < actors_.size(); ++i)
    {
        if(!actors_[i]->isExausted())
        {
            QJsonObject actorObject;
            actors_[i]->store(actorObject);
            actorsArray.append(actorObject);
        }
    }
    json["Actors"] = actorsArray;
}

void Item::load(const QJsonObject &json, const bool preserve)
{
    if(!preserve)
    {
        name_ = json["Name"].toString(name_);
        itemId_ = static_cast<uint32_t>(json["ItemId"].toDouble(0));
    }
    const QJsonArray actorsArray(json["Actors"].toArray(QJsonArray()));
    for(int i = 0; i < actorsArray.size(); ++i)
    {
        if(actorsArray[i].isObject())
        {
            Actor* actor = Actor::loadActor(actorsArray[i].toObject());
            if(actor != nullptr) addActor(actor);
        }
    }
}

void Item::setValue(uint8_t value)
{
    value_ = value;
    valueChanged(value_);
}

void Item::informValue(uint8_t value)
{
    Item::setValue(value);
}

void Item::addActor(Actor* actor)
{
    actor->setParent(this);
    actors_.push_back(actor);
    if(!secondaryFlag)
    {
        qDebug()<<"connecting actor";
        connect(actor, &Actor::sigValue, this, &Item::setValue);
    }
    connect(this, &Item::valueChanged, actor, &Actor::onValueChanged);

    SensorActor* sensorActor = dynamic_cast<SensorActor*>(actor);
    if(sensorActor != nullptr )connect(&globalSensors, &SensorStore::sensorChangedState, sensorActor, &SensorActor::sensorEvent);

    Regulator* regulator = dynamic_cast<Regulator*>(actor);
    if(regulator != nullptr )connect(&globalSensors, &SensorStore::sensorChangedState, regulator, &Regulator::sensorEvent);

    PolynomalActor* polynomalActor = dynamic_cast<PolynomalActor*>(actor);
    if(polynomalActor != nullptr )connect(&globalSensors, &SensorStore::sensorChangedState, polynomalActor, &PolynomalActor::sensorEvent);
}

bool Item::removeActor(Actor* actor)
{
    for(unsigned int i = 0; i < actors_.size(); i++)
    {
        if(actors_[i] == actor)
        {
            delete actor;
            actors_.erase(actors_.begin()+i);
            return true;
        }
    }
    return false;
}

void Item::removeAllActors()
{
    for(unsigned int i = 0; i < actors_.size(); i++) delete actors_[i];
    actors_.clear();
}

std::vector< Actor* >& Item::getActors()
{
    return actors_;
}

bool Item::hasActors()
{
    return actors_.size() > 0;
}

void Item::setActorsActive(bool in)
{
    for(unsigned i = 0; i < actors_.size(); i++) in ? actors_[i]->makeActive() : actors_[i]->makeInactive();
}
